@@ -1,6 +1,6 @@
 use crate::{
-    Builder, ByteSource, CryptoError, HasBuilder, HasIndex, TypeBuilder, TypeBuilderContainer,
-    VectorByteSource,
+    Builder, ByteSource, CryptoError, HasBuilder, HasByteSource, HasIndex, TypeBuilder,
+    TypeBuilderContainer, VectorByteSource,
 };
 use mongodb::bson::{self, Document};
 use serde::{Deserialize, Serialize};
@@ -66,6 +66,12 @@ impl HasBuilder for Data {
     }
 }
 
+impl HasByteSource for Data {
+    fn byte_source(&self) -> ByteSource {
+        self.clone().into()
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Clone, Copy)]
 #[serde(tag = "t", content = "c")]
 pub enum DataBuilder {
@@ -87,10 +93,16 @@ impl TryFrom<TypeBuilderContainer> for DataBuilder {
     }
 }
 
+impl From<DataBuilder> for TypeBuilder {
+    fn from(db: DataBuilder) -> TypeBuilder {
+        TypeBuilder::Data(db)
+    }
+}
+
 impl Builder for DataBuilder {
     type Output = Data;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
         match self {
             Self::Bool(bdb) => bdb.build(bytes),
             Self::U64(ndb) => ndb.build(bytes),
@@ -115,14 +127,26 @@ impl TryFrom<TypeBuilderContainer> for BoolDataBuilder {
     }
 }
 
+impl From<BoolDataBuilder> for TypeBuilder {
+    fn from(bdb: BoolDataBuilder) -> TypeBuilder {
+        TypeBuilder::Data(DataBuilder::Bool(bdb))
+    }
+}
+
 impl Builder for BoolDataBuilder {
     type Output = Data;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
-        let s = String::from_utf8(bytes.to_vec())
-            .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        let b = bool::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        Ok(Data::Bool(b))
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
+        match bytes {
+            Some(bytes) => {
+                let s = String::from_utf8(bytes.to_vec())
+                    .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                let b =
+                    bool::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                Ok(Data::Bool(b))
+            }
+            None => Ok(Data::Bool(false)),
+        }
     }
 }
 
@@ -140,14 +164,26 @@ impl TryFrom<TypeBuilderContainer> for U64DataBuilder {
     }
 }
 
+impl From<U64DataBuilder> for TypeBuilder {
+    fn from(ndb: U64DataBuilder) -> TypeBuilder {
+        TypeBuilder::Data(DataBuilder::U64(ndb))
+    }
+}
+
 impl Builder for U64DataBuilder {
     type Output = Data;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
-        let s = String::from_utf8(bytes.to_vec())
-            .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        let n = u64::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        Ok(Data::U64(n))
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
+        match bytes {
+            Some(bytes) => {
+                let s = String::from_utf8(bytes.to_vec())
+                    .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                let n =
+                    u64::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                Ok(Data::U64(n))
+            }
+            None => Ok(Data::U64(0)),
+        }
     }
 }
 
@@ -165,14 +201,26 @@ impl TryFrom<TypeBuilderContainer> for I64DataBuilder {
     }
 }
 
+impl From<I64DataBuilder> for TypeBuilder {
+    fn from(ndb: I64DataBuilder) -> TypeBuilder {
+        TypeBuilder::Data(DataBuilder::I64(ndb))
+    }
+}
+
 impl Builder for I64DataBuilder {
     type Output = Data;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
-        let s = String::from_utf8(bytes.to_vec())
-            .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        let n = i64::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        Ok(Data::I64(n))
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
+        match bytes {
+            Some(bytes) => {
+                let s = String::from_utf8(bytes.to_vec())
+                    .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                let n =
+                    i64::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                Ok(Data::I64(n))
+            }
+            None => Ok(Data::I64(0)),
+        }
     }
 }
 
@@ -190,14 +238,26 @@ impl TryFrom<TypeBuilderContainer> for F64DataBuilder {
     }
 }
 
+impl From<F64DataBuilder> for TypeBuilder {
+    fn from(ndb: F64DataBuilder) -> TypeBuilder {
+        TypeBuilder::Data(DataBuilder::F64(ndb))
+    }
+}
+
 impl Builder for F64DataBuilder {
     type Output = Data;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
-        let s = String::from_utf8(bytes.to_vec())
-            .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        let n = f64::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        Ok(Data::F64(n))
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
+        match bytes {
+            Some(bytes) => {
+                let s = String::from_utf8(bytes.to_vec())
+                    .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                let n =
+                    f64::from_str(&s).map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                Ok(Data::F64(n))
+            }
+            None => Ok(Data::F64(0.0)),
+        }
     }
 }
 
@@ -215,13 +275,24 @@ impl TryFrom<TypeBuilderContainer> for StringDataBuilder {
     }
 }
 
+impl From<StringDataBuilder> for TypeBuilder {
+    fn from(sdb: StringDataBuilder) -> TypeBuilder {
+        TypeBuilder::Data(DataBuilder::String(sdb))
+    }
+}
+
 impl Builder for StringDataBuilder {
     type Output = Data;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
-        let s = String::from_utf8(bytes.to_vec())
-            .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
-        Ok(Data::String(s))
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
+        match bytes {
+            Some(bytes) => {
+                let s = String::from_utf8(bytes.to_vec())
+                    .map_err(|_| CryptoError::NotDeserializableToBaseDataType)?;
+                Ok(Data::String(s))
+            }
+            None => Ok(Data::String("".to_owned())),
+        }
     }
 }
 
@@ -311,23 +382,26 @@ mod tests {
         let ds = Data::String("hello, world!".to_owned());
 
         assert_eq!(
-            db.builder().build(b"true").unwrap().to_string(),
+            db.builder().build(Some(b"true")).unwrap().to_string(),
             db.to_string()
         );
         assert_eq!(
-            du.builder().build(b"10").unwrap().to_string(),
+            du.builder().build(Some(b"10")).unwrap().to_string(),
             du.to_string()
         );
         assert_eq!(
-            di.builder().build(b"-10").unwrap().to_string(),
+            di.builder().build(Some(b"-10")).unwrap().to_string(),
             di.to_string()
         );
         assert_eq!(
-            df.builder().build(b"-10.46").unwrap().to_string(),
+            df.builder().build(Some(b"-10.46")).unwrap().to_string(),
             df.to_string()
         );
         assert_eq!(
-            ds.builder().build(b"hello, world!").unwrap().to_string(),
+            ds.builder()
+                .build(Some(b"hello, world!"))
+                .unwrap()
+                .to_string(),
             ds.to_string()
         );
     }
@@ -336,7 +410,7 @@ mod tests {
     fn test_databuilder_from_typebuildercontainer_valid() {
         let tbc = TypeBuilderContainer(TypeBuilder::Data(DataBuilder::Bool(BoolDataBuilder {})));
         let db: DataBuilder = tbc.try_into().unwrap();
-        let d = db.build(b"true").unwrap();
+        let d = db.build(Some(b"true")).unwrap();
         match d {
             Data::Bool(b) => assert_eq!(b, true),
             _ => panic!("Extracted data should have been a bool-type"),
@@ -355,7 +429,7 @@ mod tests {
     #[test]
     fn test_booldatabuilder_build_true() {
         let bdb = BoolDataBuilder {};
-        let d = bdb.build(b"true").unwrap();
+        let d = bdb.build(Some(b"true")).unwrap();
         match d {
             Data::Bool(b) => assert_eq!(b, true),
             _ => panic!("Extracted data should have been a bool-type"),
@@ -365,7 +439,7 @@ mod tests {
     #[test]
     fn test_booldatabuilder_build_false() {
         let bdb = BoolDataBuilder {};
-        let d = bdb.build(b"false").unwrap();
+        let d = bdb.build(Some(b"false")).unwrap();
         match d {
             Data::Bool(b) => assert_eq!(b, false),
             _ => panic!("Extracted data should have been a bool-type"),
@@ -390,7 +464,7 @@ mod tests {
     #[test]
     fn test_u64databuilder_build_valid() {
         let udb = U64DataBuilder {};
-        let d = udb.build(b"10").unwrap();
+        let d = udb.build(Some(b"10")).unwrap();
         match d {
             Data::U64(n) => assert_eq!(n, 10),
             _ => panic!("Extracted data should have been a u64-type"),
@@ -401,7 +475,7 @@ mod tests {
     #[should_panic]
     fn test_u64databuilder_build_invalid() {
         let udb = U64DataBuilder {};
-        udb.build(b"-10").unwrap();
+        udb.build(Some(b"-10")).unwrap();
     }
 
     #[test]
@@ -422,7 +496,7 @@ mod tests {
     #[test]
     fn test_i64databuilder_build_valid() {
         let udb = I64DataBuilder {};
-        let d = udb.build(b"-10").unwrap();
+        let d = udb.build(Some(b"-10")).unwrap();
         match d {
             Data::I64(n) => assert_eq!(n, -10),
             _ => panic!("Extracted data should have been a i64-type"),
@@ -433,7 +507,7 @@ mod tests {
     #[should_panic]
     fn test_i64databuilder_build_invalid() {
         let udb = I64DataBuilder {};
-        udb.build(b"-10.54").unwrap();
+        udb.build(Some(b"-10.54")).unwrap();
     }
 
     #[test]
@@ -454,7 +528,7 @@ mod tests {
     #[test]
     fn test_f64databuilder_build_valid() {
         let udb = F64DataBuilder {};
-        let d = udb.build(b"-10.53").unwrap();
+        let d = udb.build(Some(b"-10.53")).unwrap();
         match d {
             Data::F64(n) => assert!((n + 10.53).abs() < f64::EPSILON),
             _ => panic!("Extracted data should have been a f64-type"),
@@ -465,7 +539,7 @@ mod tests {
     #[should_panic]
     fn test_f64databuilder_build_invalid() {
         let udb = F64DataBuilder {};
-        udb.build(b"somestr").unwrap();
+        udb.build(Some(b"somestr")).unwrap();
     }
 
     #[test]
@@ -486,7 +560,7 @@ mod tests {
     #[test]
     fn test_stringdatabuilder_build_valid() {
         let sdb = StringDataBuilder {};
-        let d = sdb.build(b"hello, world!").unwrap();
+        let d = sdb.build(Some(b"hello, world!")).unwrap();
         match d {
             Data::String(s) => assert_eq!(s, "hello, world!".to_owned()),
             _ => panic!("Extracted data should have been a string-type"),
@@ -497,7 +571,7 @@ mod tests {
     #[should_panic]
     fn test_stringdatabuilder_build_invalid() {
         let udb = StringDataBuilder {};
-        udb.build(vec![0xc3, 0x28].as_ref()).unwrap();
+        udb.build(Some(vec![0xc3, 0x28].as_ref())).unwrap();
     }
 
     #[test]
@@ -505,7 +579,7 @@ mod tests {
         let tbc =
             TypeBuilderContainer(TypeBuilder::Data(DataBuilder::String(StringDataBuilder {})));
         let db: StringDataBuilder = tbc.try_into().unwrap();
-        let d = db.build(b"hello, world!").unwrap();
+        let d = db.build(Some(b"hello, world!")).unwrap();
         match d {
             Data::String(s) => assert_eq!(s, "hello, world!".to_owned()),
             _ => panic!("Extracted data should have been a string-type"),
