@@ -6,8 +6,8 @@ use self::sodiumoxide::{
     SodiumOxideSymmetricKeyBuilder,
 };
 use crate::{
-    Builder, ByteSource, ByteUnsealable, CryptoError, EntryPath, HasBuilder, HasIndex,
-    SymmetricNonce, TypeBuilder, TypeBuilderContainer,
+    Builder, ByteSource, ByteUnsealable, CryptoError, EntryPath, HasBuilder, HasByteSource,
+    HasIndex, SymmetricNonce, TypeBuilder, TypeBuilderContainer,
 };
 use mongodb::bson::{self, Document};
 use serde::{Deserialize, Serialize};
@@ -125,6 +125,15 @@ impl HasBuilder for Key {
     }
 }
 
+impl HasByteSource for Key {
+    fn byte_source(&self) -> ByteSource {
+        match self {
+            Self::Symmetric(sk) => sk.byte_source(),
+            Self::Asymmetric(ak) => ak.byte_source(),
+        }
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Copy, Clone)]
 #[serde(tag = "t", content = "c")]
 pub enum KeyBuilder {
@@ -143,10 +152,16 @@ impl TryFrom<TypeBuilderContainer> for KeyBuilder {
     }
 }
 
+impl From<KeyBuilder> for TypeBuilder {
+    fn from(kb: KeyBuilder) -> TypeBuilder {
+        TypeBuilder::Key(kb)
+    }
+}
+
 impl Builder for KeyBuilder {
     type Output = Key;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
         match self {
             Self::Symmetric(sk) => Ok(Key::Symmetric(sk.build(bytes)?)),
             Self::Asymmetric(ak) => Ok(Key::Asymmetric(ak.build(bytes)?)),
@@ -174,7 +189,7 @@ impl SymmetricSealer for SymmetricKey {
             Self::SodiumOxide(sosk) => {
                 let nonce = match nonce {
                     Some(n) => match n {
-                        SymmetricNonce::SodiumOxide(sosn) => Ok(Some(sosn)),
+                        SymmetricNonce::SodiumOxide(sosn) => Ok::<_, CryptoError>(Some(sosn)),
                     },
                     None => Ok(None),
                 }?;
@@ -213,6 +228,14 @@ impl HasBuilder for SymmetricKey {
     }
 }
 
+impl HasByteSource for SymmetricKey {
+    fn byte_source(&self) -> ByteSource {
+        match self {
+            Self::SodiumOxide(sosk) => sosk.byte_source(),
+        }
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Copy, Clone)]
 #[serde(tag = "t", content = "c")]
 pub enum SymmetricKeyBuilder {
@@ -230,10 +253,16 @@ impl TryFrom<TypeBuilderContainer> for SymmetricKeyBuilder {
     }
 }
 
+impl From<SymmetricKeyBuilder> for TypeBuilder {
+    fn from(skb: SymmetricKeyBuilder) -> TypeBuilder {
+        TypeBuilder::Key(KeyBuilder::Symmetric(skb))
+    }
+}
+
 impl Builder for SymmetricKeyBuilder {
     type Output = SymmetricKey;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
         match self {
             Self::SodiumOxide(soskb) => Ok(SymmetricKey::SodiumOxide(soskb.build(bytes)?)),
         }
@@ -275,6 +304,15 @@ impl HasBuilder for AsymmetricKey {
     }
 }
 
+impl HasByteSource for AsymmetricKey {
+    fn byte_source(&self) -> ByteSource {
+        match self {
+            Self::Public(pak) => pak.byte_source(),
+            Self::Secret(sak) => sak.byte_source(),
+        }
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Copy, Clone)]
 #[serde(tag = "t", content = "c")]
 pub enum AsymmetricKeyBuilder {
@@ -293,10 +331,16 @@ impl TryFrom<TypeBuilderContainer> for AsymmetricKeyBuilder {
     }
 }
 
+impl From<AsymmetricKeyBuilder> for TypeBuilder {
+    fn from(akb: AsymmetricKeyBuilder) -> TypeBuilder {
+        TypeBuilder::Key(KeyBuilder::Asymmetric(akb))
+    }
+}
+
 impl Builder for AsymmetricKeyBuilder {
     type Output = AsymmetricKey;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
         match self {
             Self::Public(pakb) => Ok(AsymmetricKey::Public(pakb.build(bytes)?)),
             Self::Secret(sakb) => Ok(AsymmetricKey::Secret(sakb.build(bytes)?)),
@@ -340,6 +384,14 @@ impl HasBuilder for PublicAsymmetricKey {
     }
 }
 
+impl HasByteSource for PublicAsymmetricKey {
+    fn byte_source(&self) -> ByteSource {
+        match self {
+            Self::SodiumOxide(sopak) => sopak.byte_source(),
+        }
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Copy, Clone)]
 #[serde(tag = "t", content = "c")]
 pub enum PublicAsymmetricKeyBuilder {
@@ -359,10 +411,16 @@ impl TryFrom<TypeBuilderContainer> for PublicAsymmetricKeyBuilder {
     }
 }
 
+impl From<PublicAsymmetricKeyBuilder> for TypeBuilder {
+    fn from(pakb: PublicAsymmetricKeyBuilder) -> TypeBuilder {
+        TypeBuilder::Key(KeyBuilder::Asymmetric(AsymmetricKeyBuilder::Public(pakb)))
+    }
+}
+
 impl Builder for PublicAsymmetricKeyBuilder {
     type Output = PublicAsymmetricKey;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
         match self {
             Self::SodiumOxide(sopakb) => Ok(PublicAsymmetricKey::SodiumOxide(sopakb.build(bytes)?)),
         }
@@ -405,6 +463,14 @@ impl HasBuilder for SecretAsymmetricKey {
     }
 }
 
+impl HasByteSource for SecretAsymmetricKey {
+    fn byte_source(&self) -> ByteSource {
+        match self {
+            Self::SodiumOxide(sosak) => sosak.byte_source(),
+        }
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Copy, Clone)]
 #[serde(tag = "t", content = "c")]
 pub enum SecretAsymmetricKeyBuilder {
@@ -424,10 +490,16 @@ impl TryFrom<TypeBuilderContainer> for SecretAsymmetricKeyBuilder {
     }
 }
 
+impl From<SecretAsymmetricKeyBuilder> for TypeBuilder {
+    fn from(sakb: SecretAsymmetricKeyBuilder) -> TypeBuilder {
+        TypeBuilder::Key(KeyBuilder::Asymmetric(AsymmetricKeyBuilder::Secret(sakb)))
+    }
+}
+
 impl Builder for SecretAsymmetricKeyBuilder {
     type Output = SecretAsymmetricKey;
 
-    fn build(&self, bytes: &[u8]) -> Result<Self::Output, CryptoError> {
+    fn build(&self, bytes: Option<&[u8]>) -> Result<Self::Output, CryptoError> {
         match self {
             Self::SodiumOxide(sosakb) => Ok(SecretAsymmetricKey::SodiumOxide(sosakb.build(bytes)?)),
         }
