@@ -74,6 +74,12 @@ pub use key::{
 };
 pub use nonce::{AsymmetricNonce, Nonce, SymmetricNonce};
 pub use sealable::{ByteSealable, Sealable};
-pub use source::{ByteSource, FsByteSource, HasByteSource, Path, Source, VectorByteSource};
-pub use storage::{mongodb::MongoStorer, redact::RedactStorer, HasIndex, Storer};
+pub use source::{
+    ByteSource, FsByteSource, HasByteSource, Path, Source, SourceError, VectorByteSource,
+};
+pub use storage::{
+    mongodb::{MongoStorer, MongoStorerError},
+    redact::{RedactStorer, RedactStorerError},
+    HasIndex, Storer,
+};
 pub use unsealable::{ByteUnsealable, Unsealable};
